@@ -197,6 +197,7 @@ void tangle_actor_show(TangleActor* actor) {
 		if ((animation = animate_transition(actor, NULL, current_allocation))) {
 			g_signal_connect_swapped(animation, "completed", G_CALLBACK(unset_transition_scale_children), actor);
 		}
+		clutter_actor_box_free(current_allocation);
 		
 		/* TODO: if hiding_animation should have done hide_all... */
 	}
@@ -217,6 +218,7 @@ void tangle_actor_hide_animated(TangleActor* actor) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
 			animation = animate_transition(actor, current_allocation, NULL);
+			clutter_actor_box_free(current_allocation);
 		}
 		if (animation) {
 			actor->priv->hiding_animation = animation;
@@ -243,6 +245,7 @@ void tangle_actor_hide_all_animated(TangleActor* actor) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
 			animation = animate_transition(actor, current_allocation, NULL);
+			clutter_actor_box_free(current_allocation);
 		}
 		if (animation) {
 			actor->priv->hiding_animation = animation;
@@ -271,6 +274,7 @@ void tangle_actor_destroy_animated(TangleActor* actor) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
 			animation = animate_transition(actor, current_allocation, NULL);
+			clutter_actor_box_free(current_allocation);
 		}
 		if (animation) {
 			actor->priv->hiding_animation = animation;
@@ -843,6 +847,11 @@ static void tangle_actor_allocate(ClutterActor* self, const ClutterActorBox* box
 			g_signal_connect_swapped(animation, "completed", G_CALLBACK(unset_transition_scale_children), self);
 		}
 	}
+	
+	if (old_allocation) {
+		clutter_actor_box_free(old_allocation);
+	}
+	clutter_actor_box_free(new_allocation);
 }
 
 static void tangle_actor_paint(ClutterActor* clutter_actor) {
@@ -1285,20 +1294,40 @@ static void tangle_actor_get_property(GObject* object, guint prop_id, GValue* va
         }
 }
 
-static void tangle_actor_finalize(GObject* object) {
-	TangleActor* actor;
-	
-	actor = TANGLE_ACTOR(object);
-	g_hash_table_destroy(actor->priv->animations);
-
-	G_OBJECT_CLASS (tangle_actor_parent_class)->finalize (object);
-}
-
 static void tangle_actor_dispose(GObject* object) {
 	TangleActor* actor;
 	
 	actor = TANGLE_ACTOR(object);
 
+	if (actor->priv->animations) {
+		g_hash_table_destroy(actor->priv->animations);
+		actor->priv->animations = NULL;
+	}
+
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->button_press_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->button_release_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->enter_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->leave_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->motion_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->scroll_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->hide_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->show_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->show_completed_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->transition_completed_action);
+
+	if (actor->priv->ancestor_events) {
+		g_list_foreach(actor->priv->ancestor_events, (GFunc)tangle_event_free, NULL);
+		g_list_free(actor->priv->ancestor_events);
+		actor->priv->ancestor_events = NULL;
+	}
+	if (actor->priv->descendant_events) {
+		g_list_foreach(actor->priv->descendant_events, (GFunc)tangle_event_free, NULL);
+		g_list_free(actor->priv->descendant_events);
+		actor->priv->descendant_events = NULL;
+	}
+
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->hiding_animation);
+
 	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->script);
 
 	G_OBJECT_CLASS (tangle_actor_parent_class)->dispose (object);
@@ -1309,7 +1338,6 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	ClutterActorClass* clutter_actor_class;
 	
 	gobject_class = G_OBJECT_CLASS(actor_class);
-	gobject_class->finalize = tangle_actor_finalize;
 	gobject_class->dispose = tangle_actor_dispose;
 	gobject_class->set_property = tangle_actor_set_property;
 	gobject_class->get_property = tangle_actor_get_property;
@@ -1682,6 +1710,8 @@ static void on_transition_animation_completed(ClutterAnimation* animation, gpoin
 	if (!from_allocation) {
 		g_signal_emit(actor, signals[SHOW_COMPLETED], 0, to_allocation);
 	}
+	clutter_actor_box_free(from_allocation);
+	clutter_actor_box_free(to_allocation);
 }
 
 static ClutterAnimation* animate_transition(TangleActor* actor, ClutterActorBox* from_allocation, ClutterActorBox* to_allocation) {
