@@ -8,6 +8,7 @@
  */
 
 #include "tangle-button.h"
+#include "tangle-misc.h"
 #include "marshalers.h"
 
 /**
@@ -300,11 +301,21 @@ static void tangle_button_get_property(GObject* object, guint prop_id, GValue* v
         }
 }
 
-static void tangle_button_finalize(GObject* object) {
-	G_OBJECT_CLASS(tangle_button_parent_class)->finalize(object);
-}
-
 static void tangle_button_dispose(GObject* object) {
+	TangleButton* button;
+	
+	button = TANGLE_BUTTON(object);
+	
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->normal_background_actor);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->interactive_background_actor);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->selected_background_actor);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->clicked_action);
+	
+	if (button->priv->button_release_event_handler_id) {
+		g_signal_handler_disconnect(clutter_actor_get_stage(CLUTTER_ACTOR(button)), button->priv->button_release_event_handler_id);
+		button->priv->button_release_event_handler_id = 0;
+	}
+
 	G_OBJECT_CLASS(tangle_button_parent_class)->dispose(object);
 }
 
@@ -313,7 +324,6 @@ static void tangle_button_class_init(TangleButtonClass* button_class) {
 	ClutterActorClass* clutter_actor_class = CLUTTER_ACTOR_CLASS(button_class);
 	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(button_class);
 
-	gobject_class->finalize = tangle_button_finalize;
 	gobject_class->dispose = tangle_button_dispose;
 	gobject_class->set_property = tangle_button_set_property;
 	gobject_class->get_property = tangle_button_get_property;
