@@ -78,7 +78,7 @@ TangleDragging* tangle_dragging_copy(const TangleDragging* dragging) {
 	*copy_of_dragging = *dragging;
 	g_object_ref(dragging->draggable_actor);
 	if (dragging->droppable_actor) {
-		//g_object_ref(dragging->droppable_actor);
+		g_object_ref(dragging->droppable_actor);
 	}
 	
 	return copy_of_dragging;
@@ -87,7 +87,7 @@ TangleDragging* tangle_dragging_copy(const TangleDragging* dragging) {
 void tangle_dragging_free(TangleDragging* dragging) {
 	g_object_unref(dragging->draggable_actor);
 	if (dragging->droppable_actor) {
-		//g_object_unref(dragging->droppable_actor);
+		g_object_unref(dragging->droppable_actor);
 	}
 	g_free(dragging);
 }
@@ -277,6 +277,15 @@ static void tangle_draggable_actor_finalize(GObject* object) {
 }
 
 static void tangle_draggable_actor_dispose(GObject* object) {
+	TangleDraggableActor* draggable_actor;
+	
+	draggable_actor = TANGLE_DRAGGABLE_ACTOR(object);
+	
+	if (draggable_actor->priv->dragging) {
+		tangle_dragging_free(draggable_actor->priv->dragging);
+		draggable_actor->priv->dragging = NULL;
+	}
+
 	G_OBJECT_CLASS(tangle_draggable_actor_parent_class)->dispose(object);
 }
 
