@@ -202,20 +202,10 @@ static void tangle_grid_get_property(GObject* object, guint prop_id, GValue* val
         }
 }
 
-static void tangle_grid_finalize(GObject* object) {
-	G_OBJECT_CLASS(tangle_grid_parent_class)->finalize(object);
-}
-
-static void tangle_grid_dispose(GObject* object) {
-	G_OBJECT_CLASS(tangle_grid_parent_class)->dispose(object);
-}
-
 static void tangle_grid_class_init(TangleGridClass* grid_class) {
 	GObjectClass* gobject_class = G_OBJECT_CLASS(grid_class);
 	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(grid_class);
 
-	gobject_class->finalize = tangle_grid_finalize;
-	gobject_class->dispose = tangle_grid_dispose;
 	gobject_class->set_property = tangle_grid_set_property;
 	gobject_class->get_property = tangle_grid_get_property;
 
