@@ -631,18 +631,31 @@ static void tangle_widget_get_property(GObject* object, guint prop_id, GValue* v
         }
 }
 
-static void tangle_widget_finalize(GObject* object) {
-	G_OBJECT_CLASS(tangle_widget_parent_class)->finalize(object);
-}
-
 static void tangle_widget_dispose(GObject* object) {
 	TangleWidget* widget;
 	
 	widget = TANGLE_WIDGET(object);
 	
-	/* TODO: layout */
+	if (widget->priv->children) {
+		g_list_foreach(widget->priv->children, (GFunc)clutter_actor_destroy, NULL);
+		g_list_free(widget->priv->children);
+		widget->priv->children = NULL;
+	}
+	
+	if (widget->priv->layout) {
+		tangle_layout_set_widget(widget->priv->layout, NULL);
+		g_object_unref(widget->priv->layout);
+		widget->priv->layout = NULL;
+	}
+	
 	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(widget->priv->stylesheet);
 
+	if (widget->priv->background_actor) {
+		clutter_actor_unparent(widget->priv->background_actor);
+		g_object_unref(widget->priv->background_actor);
+		widget->priv->background_actor = NULL;
+	}
+
 	G_OBJECT_CLASS(tangle_widget_parent_class)->dispose(object);
 }
 
@@ -651,7 +664,6 @@ static void tangle_widget_class_init(TangleWidgetClass* widget_class) {
 	ClutterActorClass* clutter_actor_class = CLUTTER_ACTOR_CLASS(widget_class);
 	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(widget_class);
 	
-	gobject_class->finalize = tangle_widget_finalize;
 	gobject_class->dispose = tangle_widget_dispose;
 	gobject_class->set_property = tangle_widget_set_property;
 	gobject_class->get_property = tangle_widget_get_property;
